import 'dart:async';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/foundation.dart';
import 'package:get/get.dart';
import 'dart:io';
import 'package:wowit/common.dart';
import 'package:wowit/firebase_service.dart';
import 'package:wowit/interfaces/mixins/online_status_mixin.dart';

class WowConnectivity {
  WowConnectivity._();

  factory WowConnectivity() => _instance;

  static final WowConnectivity _instance = WowConnectivity._();

  static final Map<OnlineStatusMixin, StreamSubscription<ConnectivityResult>>
      _listSubscription = {};

  bool _isOnline = true;
  bool get isOnline => _isOnline;

  final Connectivity _connectivity = Connectivity();

  void initialise(OnlineStatusMixin controller) async {
    var result = await _connectivity.checkConnectivity();
    _checkStatus(result, controller);
    _listSubscription[controller] =
        _connectivity.onConnectivityChanged.listen((connectivityResult) {
      _checkStatus(connectivityResult, controller);
    });
  }

  void dispose(OnlineStatusMixin baseController) {
    if (_listSubscription.containsKey(baseController)) {
      _listSubscription[baseController]!.cancel();
    }
  }

  void _checkStatus(ConnectivityResult connectivityResult,
      OnlineStatusMixin controller) async {
    bool isOnline = true;
    try {
      final result = await InternetAddress.lookup('google.com');
      if (connectivityResult != ConnectivityResult.none &&
          result.isNotEmpty &&
          result[0].rawAddress.isNotEmpty) {
        try {
          isOnline = true;
        } catch (e) {
          if (kDebugMode) {
            print('Exception _checkStatus: ' + e.toString());
          }
          await FirebaseService.logout();
          await Get.offAll(() => Common.registerScreen);
        }
      } else {
        isOnline = false;
      }
    } on SocketException catch (e) {
      if (kDebugMode) {
        print('Exception _checkStatus: ' + e.toString());
      }
      isOnline = false;
    }
    if (_isOnline != isOnline) {
      _isOnline = isOnline;
    }
    if (kDebugMode) {
      print(
          'Connectivity reload connect ${controller.runtimeType}: isOnline = $_isOnline');
    }
    controller.isOnline = isOnline;
  }
}
